.content.svelte-vgp2pk.svelte-vgp2pk{overflow:auto}.form-grid.svelte-vgp2pk.svelte-vgp2pk{display:grid;grid-template-columns:1fr 2fr;column-gap:0.5rem;row-gap:0.75rem}.form-grid.svelte-vgp2pk label.svelte-vgp2pk,.form-grid.svelte-vgp2pk .label-like.svelte-vgp2pk{line-height:1.75em;color:var(--inverse-color-tone-a);text-align:right}.section-header.svelte-vgp2pk.svelte-vgp2pk{margin-bottom:1rem;padding-bottom:0.5rem;border-bottom:1px solid var(--inverse-color-tone-b);font-weight:500;text-align:center}.value-rows.svelte-vgp2pk.svelte-vgp2pk{display:flex;flex-direction:column;gap:0.25rem}.description.svelte-vgp2pk.svelte-vgp2pk{font-size:0.875em;color:var(--inverse-color-tone-b)}
.modal-overlay.svelte-i0n924{position:fixed;top:0;bottom:0;left:0;right:0;z-index:30}.modal-overlay.svelte-i0n924{display:flex;align-items:center;justify-content:center}.modal-wrapper.svelte-i0n924{display:contents}.modal.svelte-i0n924{display:flex;flex-direction:column;max-height:75%}
.modal.svelte-1lyvqkk{position:absolute;top:calc(100% + 4px);left:0;right:0;z-index:10}
.profile-switcher-wrapper.svelte-ys6f4f.svelte-ys6f4f{position:relative}.profile-switcher.svelte-ys6f4f.svelte-ys6f4f{display:flex;font-size:0.95em}.profile-switcher.svelte-ys6f4f>.svelte-ys6f4f:not(:first-child){margin-left:4px}.profile-switcher.svelte-ys6f4f button.svelte-ys6f4f,.profile-switcher.svelte-ys6f4f select.svelte-ys6f4f{line-height:2em;margin:0;padding:0 0.5em;border:0;border-radius:0.3333333333em;transition:all 0.2s;font-weight:500}.profile-switcher.svelte-ys6f4f button.svelte-ys6f4f:disabled,.profile-switcher.svelte-ys6f4f select.svelte-ys6f4f:disabled{opacity:0.5}.profile-switcher.svelte-ys6f4f button.svelte-ys6f4f:hover:not(:disabled),.profile-switcher.svelte-ys6f4f select.svelte-ys6f4f:hover:not(:disabled){background-color:var(--inverse-color-alpha-05)}.profile-switcher.svelte-ys6f4f button.svelte-ys6f4f:focus,.profile-switcher.svelte-ys6f4f select.svelte-ys6f4f:focus{background-color:var(--inverse-color-alpha-20)}.profile-switcher.svelte-ys6f4f button.svelte-ys6f4f{display:flex;place-items:center}.profile-switcher.svelte-ys6f4f select.svelte-ys6f4f{padding-left:0.65em;padding-right:1.25rem;box-shadow:inset 0 0 0 1px var(--inverse-color-alpha-10);background-image:var(--dropdown-arrow);background-size:10px;background-repeat:no-repeat;background-position:top calc(50% + 1px) right 7px;min-width:180px}.profile-switcher.svelte-ys6f4f .svelte-ys6f4f:first-child{border-top-left-radius:0.5em;border-bottom-left-radius:0.5em}.profile-switcher.svelte-ys6f4f .svelte-ys6f4f:last-child{border-top-right-radius:0.5em;border-bottom-right-radius:0.5em}.profile-switcher.svelte-ys6f4f .icon{height:1rem;width:1rem;vertical-align:text-bottom;fill:var(--inverse-color-tone-b)}
form.svelte-1074j9 .start-date-suggestions.svelte-1074j9{list-style:none;margin:0;padding-left:0}form.svelte-1074j9 .form-grid.svelte-1074j9{display:grid;grid-template-columns:1fr 2fr;column-gap:0.5rem;row-gap:0.5rem}form.svelte-1074j9 label.svelte-1074j9{line-height:1.75em;color:var(--inverse-color-tone-a);text-align:right}form.svelte-1074j9 .start-date-suggestions.svelte-1074j9{display:inline-flex;gap:0.2rem;grid-column-start:2;margin-top:-0.3em;font-size:0.8em;font-weight:600}form.svelte-1074j9 .start-date-suggestions li button.svelte-1074j9{cursor:pointer;margin:0;padding:0.2rem 0.4rem;border:0;border-radius:0.2916666667em;background-color:var(--inverse-color-tone-b);color:var(--base-color)}form.svelte-1074j9 .start-date-suggestions li.disabled button.svelte-1074j9{cursor:default;opacity:0.5}form.svelte-1074j9 .start-date-suggestions li:first-child button.svelte-1074j9{border-top-left-radius:0.4375em;border-bottom-left-radius:0.4375em}form.svelte-1074j9 .start-date-suggestions li:last-child button.svelte-1074j9{border-top-right-radius:0.4375em;border-bottom-right-radius:0.4375em}.form-control.checkbox-container.svelte-1074j9.svelte-1074j9{background-color:transparent;padding-left:0}
.content.svelte-n5l6oa{overflow:scroll;-webkit-user-select:text;user-select:text}.content.svelte-n5l6oa pre{white-space:pre-wrap}.content.svelte-n5l6oa h1{font-size:1.4em}.content.svelte-n5l6oa h2{font-size:1.2em}.content.svelte-n5l6oa h3{font-size:1em}.content.svelte-n5l6oa .link{word-break:break-all}
.choices.svelte-1xsfw74.svelte-1xsfw74.svelte-1xsfw74{list-style:none;margin:0;padding-left:0}.addon-stub.svelte-1xsfw74.svelte-1xsfw74.svelte-1xsfw74{transition:all 0.2s}.header.svelte-1xsfw74.svelte-1xsfw74.svelte-1xsfw74,.selection-controls.svelte-1xsfw74.svelte-1xsfw74.svelte-1xsfw74{padding:0.4em 0.75em}.header.svelte-1xsfw74.svelte-1xsfw74.svelte-1xsfw74{display:flex;flex-direction:row;align-items:baseline;justify-content:space-between;gap:1rem}.folders.svelte-1xsfw74.svelte-1xsfw74.svelte-1xsfw74{overflow-x:hidden;white-space:nowrap;text-overflow:ellipsis;font-weight:700}.folders.svelte-1xsfw74.svelte-1xsfw74.svelte-1xsfw74:only-child{line-height:1.5rem}.folders.svelte-1xsfw74 .remaining-folders.svelte-1xsfw74.svelte-1xsfw74{font-size:0.8em;color:var(--inverse-color-tone-b)}.defn-or-version.svelte-1xsfw74.svelte-1xsfw74.svelte-1xsfw74,.choices.svelte-1xsfw74 li.svelte-1xsfw74.svelte-1xsfw74{font-family:Menlo, Monaco, Consolas, monospace;font-size:0.7rem}.selection-controls.svelte-1xsfw74.svelte-1xsfw74.svelte-1xsfw74{margin-top:-0.25rem;padding-top:0;color:var(--inverse-color-tone-b)}.selection-controls.svelte-1xsfw74 .selection-grid.svelte-1xsfw74.svelte-1xsfw74{display:grid;grid-template-columns:1fr 2fr 1rem;column-gap:0.5rem}.selection-controls.svelte-1xsfw74 summary.svelte-1xsfw74.svelte-1xsfw74{list-style:none;line-height:1rem}.selection-controls.svelte-1xsfw74 summary.svelte-1xsfw74.svelte-1xsfw74::-webkit-details-marker,.selection-controls.svelte-1xsfw74 summary.svelte-1xsfw74.svelte-1xsfw74::marker{display:none}.selection-controls.svelte-1xsfw74 summary .svelte-1xsfw74.svelte-1xsfw74:nth-child(2){padding:0 0.2rem}.selection-controls.svelte-1xsfw74 summary .svelte-1xsfw74.svelte-1xsfw74:last-child{justify-self:right}.selection-controls.svelte-1xsfw74 summary.svelte-1xsfw74 .icon{display:block;height:1rem;width:1rem;fill:var(--inverse-color-tone-b)}.selection-controls[open].svelte-1xsfw74 .icon-collapsed{display:none !important}.selection-controls.svelte-1xsfw74:not([open]) .icon-expanded{display:none !important}.choices.svelte-1xsfw74 li.svelte-1xsfw74.svelte-1xsfw74{display:flex;padding:0 0.2rem;grid-column-start:2;line-height:1rem}.choices.svelte-1xsfw74 li.svelte-1xsfw74.svelte-1xsfw74:first-child{margin-top:0.18rem;padding-top:0.18rem;border-top:1px solid var(--inverse-color-tone-b)}.choices.svelte-1xsfw74 li label.svelte-1xsfw74.svelte-1xsfw74{flex-grow:1}.choices.svelte-1xsfw74 li label.svelte-1xsfw74.svelte-1xsfw74::before{content:"( ) "}.choices.svelte-1xsfw74 [type=radio].svelte-1xsfw74.svelte-1xsfw74{display:none}.choices.svelte-1xsfw74 [type=radio].svelte-1xsfw74:checked+label.svelte-1xsfw74::before{content:"(x) "}.unreconciled-message.svelte-1xsfw74.svelte-1xsfw74.svelte-1xsfw74{margin:0;font-size:0.75em;color:var(--inverse-color-tone-b)}
svg.svelte-1iz2oiq{transform:rotate(-90deg)}circle.svelte-1iz2oiq{transition:stroke-dashoffset 0.35s;transform-origin:center;fill:transparent}.indeterminate.svelte-1iz2oiq{animation-duration:0.75s;animation-iteration-count:infinite;animation-name:svelte-1iz2oiq-rotate}@keyframes svelte-1iz2oiq-rotate{from{transform:rotate(0)}to{transform:rotate(1turn)}}
menu.svelte-nmk0ob.svelte-nmk0ob.svelte-nmk0ob{list-style:none;margin:0;padding-left:0}.addon-list-nav.svelte-nmk0ob.svelte-nmk0ob.svelte-nmk0ob{display:grid;column-gap:1em;place-items:center}.addon-list-nav.svelte-nmk0ob .svelte-nmk0ob.svelte-nmk0ob:first-child{justify-self:left}.addon-list-nav.svelte-nmk0ob .svelte-nmk0ob.svelte-nmk0ob:last-child{justify-self:right}.control-set.svelte-nmk0ob.svelte-nmk0ob.svelte-nmk0ob,.addon-list-nav.svelte-nmk0ob>div.svelte-nmk0ob.svelte-nmk0ob{display:flex;align-items:center}menu.svelte-nmk0ob.svelte-nmk0ob.svelte-nmk0ob{font-weight:500}.addon-list-nav.svelte-nmk0ob.svelte-nmk0ob.svelte-nmk0ob{grid-template-columns:repeat(3, 1fr);height:3rem}.search.svelte-nmk0ob .search-control.svelte-nmk0ob.svelte-nmk0ob{border-top-right-radius:0.46875em;border-bottom-right-radius:0.46875em;box-shadow:inset 0 0 0 1px var(--inverse-color-alpha-10);font-size:1rem}.search.svelte-nmk0ob .search-control.svelte-nmk0ob.svelte-nmk0ob,.search.svelte-nmk0ob .search-control.svelte-nmk0ob.svelte-nmk0ob::-webkit-search-cancel-button{-webkit-appearance:none;appearance:none}.search.svelte-nmk0ob .search-control.svelte-nmk0ob.svelte-nmk0ob:not(:focus){text-align:center}.progress-indicator.svelte-nmk0ob.svelte-nmk0ob.svelte-nmk0ob{height:18px;line-height:18px;margin:0 0.5rem}.progress-indicator.svelte-nmk0ob circle{stroke:currentColor}.control.svelte-nmk0ob.svelte-nmk0ob.svelte-nmk0ob{display:block;min-width:min-content;border:0;transition:all 0.2s;line-height:1.875em;margin:0;padding:0 0.7em;border-radius:0.3125em;white-space:nowrap}.control.svelte-nmk0ob.svelte-nmk0ob.svelte-nmk0ob:disabled{opacity:0.5}.control.svelte-nmk0ob.svelte-nmk0ob.svelte-nmk0ob:hover:not(:disabled){background-color:var(--inverse-color-alpha-05)}.control.svelte-nmk0ob.svelte-nmk0ob.svelte-nmk0ob:focus{background-color:var(--inverse-color-alpha-10)}.control[type=checkbox].svelte-nmk0ob.svelte-nmk0ob.svelte-nmk0ob,.control[type=radio].svelte-nmk0ob.svelte-nmk0ob.svelte-nmk0ob{position:absolute;opacity:0}.control[type=checkbox].svelte-nmk0ob:checked+label.svelte-nmk0ob.svelte-nmk0ob,.control[type=radio].svelte-nmk0ob:checked+label.svelte-nmk0ob.svelte-nmk0ob{background-color:var(--inverse-color-tone-b) !important;color:var(--base-color)}.control[type=checkbox].svelte-nmk0ob:checked+label.svelte-nmk0ob .icon,.control[type=radio].svelte-nmk0ob:checked+label.svelte-nmk0ob .icon{fill:var(--base-color)}.control[type=checkbox].svelte-nmk0ob:disabled+label.svelte-nmk0ob.svelte-nmk0ob,.control[type=radio].svelte-nmk0ob:disabled+label.svelte-nmk0ob.svelte-nmk0ob{opacity:0.5}.control[type=checkbox].svelte-nmk0ob:focus+label.svelte-nmk0ob.svelte-nmk0ob,.control[type=radio].svelte-nmk0ob:focus+label.svelte-nmk0ob.svelte-nmk0ob{background-color:var(--inverse-color-alpha-10)}.control.dirty.svelte-nmk0ob.svelte-nmk0ob.svelte-nmk0ob{background-image:repeating-linear-gradient(-45deg, transparent 0 20px, rgba(250, 128, 114, 0.5) 20px 40px)}.control.reconciliation-stage-control.svelte-nmk0ob.svelte-nmk0ob.svelte-nmk0ob{width:100%;padding-right:1.4rem;background-image:var(--dropdown-arrow);background-size:10px;background-repeat:no-repeat;background-position:top calc(50% + 1px) right 7px}.control.svelte-nmk0ob .icon{height:1rem;width:1rem;vertical-align:text-bottom;fill:var(--inverse-color-tone-b)}.control-set.svelte-nmk0ob.svelte-nmk0ob.svelte-nmk0ob{font-size:0.85em}.control-set.svelte-nmk0ob li.svelte-nmk0ob.svelte-nmk0ob:not(:first-child){margin-left:4px}.control-set.svelte-nmk0ob li.segmented-control.svelte-nmk0ob~.segmented-control.svelte-nmk0ob{margin-left:-1px}.control-set.svelte-nmk0ob li:first-child .control.svelte-nmk0ob.svelte-nmk0ob{border-top-left-radius:0.46875em;border-bottom-left-radius:0.46875em}.control-set.svelte-nmk0ob li:last-child .control.svelte-nmk0ob.svelte-nmk0ob{border-top-right-radius:0.46875em;border-bottom-right-radius:0.46875em}
.context-menu.svelte-1irf7oq menu.svelte-1irf7oq{list-style:none;margin:0;padding-left:0}.context-menu-wrapper.svelte-1irf7oq.svelte-1irf7oq{position:fixed;top:0;bottom:0;left:0;right:0;z-index:30}.context-menu.svelte-1irf7oq.svelte-1irf7oq{-webkit-backdrop-filter:blur(6px);backdrop-filter:blur(6px)}.context-menu.svelte-1irf7oq.svelte-1irf7oq{border-radius:0.75em;box-shadow:0 1rem 3rem var(--inverse-color-alpha-10)}.context-menu.svelte-1irf7oq.svelte-1irf7oq{position:fixed;top:var(--menu-y-offset);left:var(--menu-x-offset);z-index:40;padding:0.5em 0;background-color:var(--base-color-alpha-65);cursor:default;white-space:nowrap}.context-menu.svelte-1irf7oq menu.svelte-1irf7oq{font-size:0.8em}
.menu-item.svelte-1gyrn85 button.svelte-1gyrn85{margin:0;padding:0 0.8rem;border:0;line-height:1.38rem;width:100%;text-align:left}.menu-item.svelte-1gyrn85 button.svelte-1gyrn85:focus,.menu-item.svelte-1gyrn85 button.svelte-1gyrn85:hover{background-color:rgb(24, 136, 255);color:#efefef}.menu-item.svelte-1gyrn85 hr.svelte-1gyrn85{margin:0.4rem;border:1px solid var(--inverse-color-alpha-20)}
.addon-details.svelte-n5lp8.svelte-n5lp8.svelte-n5lp8,.addon-actions.svelte-n5lp8.svelte-n5lp8.svelte-n5lp8{list-style:none;margin:0;padding-left:0}.addon.svelte-n5lp8.svelte-n5lp8.svelte-n5lp8{position:relative;display:flex;padding:0.4em 0.75em;border-radius:inherit;transition:all 0.2s}.addon.status-being-modified.svelte-n5lp8.svelte-n5lp8.svelte-n5lp8{pointer-events:none}.addon.status-outdated.svelte-n5lp8.svelte-n5lp8.svelte-n5lp8{background-image:repeating-linear-gradient(-45deg, transparent 0 20px, rgba(0, 255, 0, 0.1) 20px 40px)}.addon.status-pinned.svelte-n5lp8.svelte-n5lp8.svelte-n5lp8{background-image:repeating-linear-gradient(-45deg, transparent 0 20px, rgba(255, 215, 0, 0.1) 20px 40px)}.addon-details.svelte-n5lp8.svelte-n5lp8.svelte-n5lp8{flex-grow:1;align-items:baseline;gap:0.5em;overflow-x:hidden}.addon-details.two-col.svelte-n5lp8.svelte-n5lp8.svelte-n5lp8{display:flex;line-height:1.5rem}.addon-details.two-col.svelte-n5lp8 .name.svelte-n5lp8.svelte-n5lp8{flex-grow:1}.addon-details.two-col.svelte-n5lp8 .versions.svelte-n5lp8.svelte-n5lp8{margin-top:0}.addon-details.svelte-n5lp8 .name.svelte-n5lp8.svelte-n5lp8{font-weight:700;white-space:nowrap;text-overflow:ellipsis;overflow-x:hidden}.addon-details.svelte-n5lp8 .versions.svelte-n5lp8.svelte-n5lp8{float:right;text-align:right}.addon-details.svelte-n5lp8 .date.svelte-n5lp8.svelte-n5lp8{white-space:nowrap}.addon-details.svelte-n5lp8 .defn.svelte-n5lp8.svelte-n5lp8,.addon-details.svelte-n5lp8 .versions.svelte-n5lp8.svelte-n5lp8{margin-top:0.25rem;font-family:Menlo, Monaco, Consolas, monospace;font-size:0.7em;color:var(--inverse-color-tone-b)}.addon-details.svelte-n5lp8 .description.svelte-n5lp8.svelte-n5lp8{margin-top:0.25rem;padding-left:0.25rem;overflow-x:hidden;white-space:nowrap;text-overflow:ellipsis;font-size:0.8em}.addon-details.svelte-n5lp8:hover .description.svelte-n5lp8.svelte-n5lp8{display:block}.addon-actions.svelte-n5lp8.svelte-n5lp8.svelte-n5lp8{display:flex;flex-wrap:nowrap;align-self:center;padding-left:0.5rem}.addon-actions.svelte-n5lp8 li.svelte-n5lp8+li.svelte-n5lp8{margin-left:4px}.addon-actions.svelte-n5lp8 li:first-child button.svelte-n5lp8.svelte-n5lp8{border-top-left-radius:1em;border-bottom-left-radius:1em}.addon-actions.svelte-n5lp8 li:last-child button.svelte-n5lp8.svelte-n5lp8{border-top-right-radius:1em;border-bottom-right-radius:1em}.addon-actions.svelte-n5lp8 button.svelte-n5lp8.svelte-n5lp8{margin:0;padding:0 0.625rem;height:1.8em;line-height:1.8em;font-size:0.8em;font-weight:600;border:0;border-radius:0.75em;background-color:rgb(24, 136, 255);color:#efefef;transition:all 0.2s}.addon-actions.svelte-n5lp8 button.svelte-n5lp8.svelte-n5lp8:disabled{opacity:0.5}.addon-actions.svelte-n5lp8 button.svelte-n5lp8.svelte-n5lp8:focus{background-color:rgb(0, 104, 217)}.addon-actions.svelte-n5lp8 button.svelte-n5lp8 .icon{display:block;width:0.8rem;height:0.8rem;fill:#efefef}.progress-indicator.svelte-n5lp8.svelte-n5lp8.svelte-n5lp8{align-self:center;justify-self:right;margin-left:0.75em}.progress-indicator.svelte-n5lp8 circle{stroke:rgb(24, 136, 255)}
.addon-list.svelte-1b3u0f7.svelte-1b3u0f7.svelte-1b3u0f7{list-style:none;margin:0;padding-left:0}.addon-list-wrapper.svelte-1b3u0f7.svelte-1b3u0f7.svelte-1b3u0f7{display:flex;flex-direction:column;flex:1;overflow-y:hidden}.addon-list-nav-wrapper.svelte-1b3u0f7.svelte-1b3u0f7.svelte-1b3u0f7{-webkit-backdrop-filter:blur(6px);backdrop-filter:blur(6px)}.placeholder.svelte-1b3u0f7.svelte-1b3u0f7.svelte-1b3u0f7{display:flex;flex-grow:1;place-items:center}.placeholder.svelte-1b3u0f7>div.svelte-1b3u0f7.svelte-1b3u0f7{flex-grow:1;text-align:center}.addon-list-wrapper.svelte-1b3u0f7.svelte-1b3u0f7.svelte-1b3u0f7{position:relative;overflow-y:auto;-webkit-user-select:none;user-select:none}.addon-list-nav-wrapper.svelte-1b3u0f7.svelte-1b3u0f7.svelte-1b3u0f7,.addon-list.svelte-1b3u0f7.svelte-1b3u0f7.svelte-1b3u0f7{padding:0 0.8em}.addon-list-nav-wrapper.svelte-1b3u0f7.svelte-1b3u0f7.svelte-1b3u0f7{background-color:var(--base-color-tone-a-alpha-85);box-shadow:inset 0 -1px 0px 0 var(--base-color-tone-b);position:sticky;top:0;z-index:20}.preamble.svelte-1b3u0f7.svelte-1b3u0f7.svelte-1b3u0f7{display:grid;grid-template-columns:3rem 1fr;grid-column-gap:0.8rem;align-items:center;margin-top:0.4rem;padding:0 1.6rem;font-size:0.85em;background-image:linear-gradient(45deg, rgba(255, 192, 203, 0.2), rgba(255, 165, 0, 0.2));color:var(--inverse-color-tone-a)}.preamble.svelte-1b3u0f7 p.svelte-1b3u0f7.svelte-1b3u0f7{margin:0.75rem 0}.preamble.svelte-1b3u0f7 .icon{width:3rem;height:3rem;fill:var(--inverse-color-tone-b)}.addon-list.svelte-1b3u0f7.svelte-1b3u0f7.svelte-1b3u0f7{margin:0.8em 0}.addon-list.svelte-1b3u0f7 li.svelte-1b3u0f7.svelte-1b3u0f7{border-radius:4px}.addon-list.svelte-1b3u0f7 li.svelte-1b3u0f7+li.svelte-1b3u0f7{margin-top:4px}.addon-list.svelte-1b3u0f7 li.svelte-1b3u0f7.svelte-1b3u0f7:nth-child(odd){background-color:var(--base-color)}
.alerts.svelte-jtijg8.svelte-jtijg8{-webkit-backdrop-filter:blur(6px);backdrop-filter:blur(6px)}.alerts.svelte-jtijg8.svelte-jtijg8{border-radius:0.75em;box-shadow:0 1rem 3rem var(--inverse-color-alpha-10)}.alerts-wrapper.svelte-jtijg8.svelte-jtijg8{position:fixed;left:0;right:0;display:flex;justify-content:center;margin-top:calc(3rem + 4px);z-index:20}.alerts.svelte-jtijg8.svelte-jtijg8{max-width:75vw;padding:0 0.8rem;background-color:var(--alert-background-color);font-size:0.8em;display:grid;grid-template-columns:auto 6rem;grid-column-gap:0.8rem;word-break:break-all}.current-alert.svelte-jtijg8 h1.svelte-jtijg8{margin-bottom:0}.alert-nav.svelte-jtijg8.svelte-jtijg8{align-self:center;justify-self:center}.alert-nav.svelte-jtijg8 button.svelte-jtijg8{line-height:1.8em;margin:0;padding:0.5em;border-radius:0.45em;border:0;transition:all 0.2s}.alert-nav.svelte-jtijg8 button.svelte-jtijg8:disabled{opacity:0.5}.alert-nav.svelte-jtijg8 button.svelte-jtijg8:hover:not(:disabled){background-color:var(--inverse-color-alpha-05)}.alert-nav.svelte-jtijg8 button.svelte-jtijg8:focus{background-color:var(--inverse-color-alpha-10);box-shadow:inset 0 0 0 1px var(--inverse-color-alpha-20)}.alert-nav.svelte-jtijg8 .icon{display:block;height:1rem;width:1rem;fill:var(--inverse-color)}
.main.svelte-tax5nz.svelte-tax5nz{display:flex;flex-direction:column;flex:1;overflow-y:hidden}.wrapper.svelte-tax5nz.svelte-tax5nz{display:flex;flex-direction:column;height:100vh}.menubar.svelte-tax5nz.svelte-tax5nz{display:flex;align-items:center;min-height:55px}.menubar.svelte-tax5nz .instawow-version.svelte-tax5nz{flex-grow:1;text-align:right;font-family:Menlo, Monaco, Consolas, monospace;font-size:0.7em}.menubar.svelte-tax5nz .instawow-version span.svelte-tax5nz{color:var(--inverse-color-tone-a)}.main.svelte-tax5nz.svelte-tax5nz{padding:0;z-index:5;background-color:var(--base-color-tone-a);border-top:4px solid var(--random-border-color)}.menubar.svelte-tax5nz.svelte-tax5nz,.statusbar.svelte-tax5nz.svelte-tax5nz{padding:0 0.8em;-webkit-user-select:none;user-select:none}.statusbar.svelte-tax5nz.svelte-tax5nz{padding-top:0.5em;padding-bottom:0.5em;background-color:var(--base-color);box-shadow:inset 0 1px var(--base-color-tone-b)}.statusbar.svelte-tax5nz .status.svelte-tax5nz{font-size:0.8em;color:var(--inverse-color-tone-a)}
